#ifndef Slash_smanip_h
#define Slash_smanip_h

#include <vector>
#include <string>

namespace Slash {

inline std::string sout(const std::string& a_string) {
  return std::string("\"")+a_string+"\"";
}

inline std::string suffix(const std::string& a_string,bool a_back = true) {
  // If a_string = dir0/dir1/dir2/dir3/name.xxx
  //   return xxx
  std::string::size_type pos = a_back?a_string.rfind('.'):a_string.find('.');
  if(pos==std::string::npos) return "";
  pos++;
  return a_string.substr(pos,a_string.size()-pos);
}

inline std::string nosuffix(const std::string& a_string,bool a_back = true){
  // If a_string = dir0/dir1/dir2/dir3/name.xxx
  //   return name
  // Start searching after the last / (or last \ for Windows).
  std::string::size_type pos = a_string.rfind('/');
  if(pos==std::string::npos) pos = a_string.rfind('\\');
  if(pos==std::string::npos) pos = 0;
  else pos++;
  std::string s = a_string.substr(pos,a_string.size()-pos);
  std::string::size_type dot_pos = a_back?s.rfind('.'):s.find('.');
  if(dot_pos==std::string::npos) return s;
  return s.substr(0,dot_pos);
}

inline void words(const std::string& a_string,const std::string& a_sep,bool a_take_empty,std::vector<std::string>& a_words){
  //  If a_sep is for exa "|" and for "xxx||xxx" :
  //  - a_take_empty false : {"xxx","xxx"} will be created 
  //    (and NOT {"xxx","","xxx"}).
  //  - a_take_empty true : {"xxx","","xxx"} will be created.
  a_words.clear();
  if(a_string.empty()) return;
  std::string::size_type lim = (a_take_empty?0:1);
  if(a_sep.empty()) {
    a_words.push_back(a_string);
  } else {
    std::string::size_type l = a_string.length();
    std::string::size_type llimiter = a_sep.length();
    std::string::size_type pos = 0;
    while(true) {
      std::string::size_type index = a_string.find(a_sep,pos);
      if(index==std::string::npos){ // Last word.
        if((l-pos)>=lim) a_words.push_back(a_string.substr(pos,l-pos));
        break;
      } else {
        //     abcxxxef
        //     0  3  67
        if((index-pos)>=lim) a_words.push_back(a_string.substr(pos,index-pos));
        pos = index + llimiter;
      }
    }
  }
}

inline std::vector<std::string> words(const std::string& a_string,const std::string& a_limiter,bool a_take_empty = false){
  std::vector<std::string> v;
  words(a_string,a_limiter,a_take_empty,v);
  return v;
}

inline bool replace(std::string& a_string,const std::string& a_old,const std::string& a_new){
  // return true : some replacement done.
  // return false : nothing replaced.
  if(a_old.empty()) return false;
  std::string snew;
  std::string::size_type lold = a_old.length();
  bool status = false;
  std::string stmp = a_string;
  while(true) {
    std::string::size_type pos = stmp.find(a_old);
    if(pos==std::string::npos){
      snew += stmp;
      break;
    } else {
      snew += stmp.substr(0,pos);
      snew += a_new;
      stmp = stmp.substr(pos+lold,stmp.length()-(pos+lold));
      status = true;
    }
  }
  a_string = snew;
  return status;
}

inline void replace(std::string& a_string,char a_old,char a_new){
  for(std::string::iterator it=a_string.begin();it!=a_string.end();++it) {
    if((*it)==a_old) *it = a_new;
  }
}

inline bool path_name_suffix(
 const std::string& a_string
,std::string& a_path
,std::string& a_name
,std::string& a_suffix
){
  // If a_string = dir0/dir1/dir2/dir3/name.xxx
  //   a_path = dir0/dir1/dir2/dir3
  //   a_name = name.xxx
  //   a_suffix = xxx
  // If a_string = dir0/name.xxx
  //   a_path = dir0
  //   a_name = name.xxx
  //   a_suffix = xxx
  // If a_string = name.xxx
  //   a_path = ""
  //   a_name = name.xxx
  //   a_suffix = xxx
  // If a_string = /name.xxx
  //   a_path = "/"
  //   a_name = name.xxx
  //   a_suffix = xxx
  // If a_string = .
  //   a_path = "."
  //   a_name = ""
  //   a_suffix = ""
  // If a_string = ..
  //   a_path = ".."
  //   a_name = ""
  //   a_suffix = ""
  if(a_string==".") {
    a_path = ".";
    a_name = "";
    a_suffix = "";
    return true;
  } else if(a_string=="..") {
    a_path = "..";
    a_name = "";
    a_suffix = "";
    return true;
  }

  std::string::size_type pos_slash = a_string.rfind('/');
  std::string::size_type pos_bslash = a_string.rfind('\\');
  std::string::size_type pos = 0;
  if(pos_slash==std::string::npos) {
    if(pos_bslash==std::string::npos) {
      pos = std::string::npos;
    } else {
      pos = pos_bslash;
    }
  } else {
    if(pos_bslash==std::string::npos) {
      pos = pos_slash;
    } else {
      if(pos_slash<=pos_bslash) {
        pos = pos_bslash;
      } else {
        pos = pos_slash;
      }
    }
  }

  if(pos==std::string::npos) {
    a_path = "";
    pos = 0;
  } else if(pos==0) {
    a_path = "/";
    pos++;
  } else {
    a_path = a_string.substr(0,pos);
    pos++;
  }
  std::string s = a_string.substr(pos,a_string.size()-pos);
  pos = s.rfind('.');
  if(pos==std::string::npos) {
    a_name = s;
    a_suffix = "";
  } else {
    a_name = s;
    pos++;
    a_suffix = s.substr(pos,s.size()-pos);
  }
  return true;
}

inline bool remove(std::vector<std::string>& a_strings,const std::string& a_string){
  bool found_some = false;
  std::vector<std::string>::iterator it;
  for(it=a_strings.begin();it!=a_strings.end();) {
    if(*it==a_string) {
      it = a_strings.erase(it);
      found_some = true;
    } else {
      ++it;
    }
  }
  return found_some;
}

inline std::string dir_name(const std::string& a_path,unsigned int a_num = 1){
  std::string path = a_path;
  for(unsigned int index=0;index<a_num;index++) {
    std::string p,n,s;
    path_name_suffix(path,p,n,s);
    path = p;
  }
  return path;
}

inline std::string base_name(const std::string& a_path) {
  std::string path,name,suffix;
  path_name_suffix(a_path,path,name,suffix);
  return name;
}

inline bool is_absolute_path(const std::string& a_path) {
  if(a_path.find('\\')!=std::string::npos) { //Windows path.
    if(a_path.find(':')!=std::string::npos) return true;
    return (a_path.size()&&(a_path[0]=='\\')?true:false);
  } else { //UNIX path
    return (a_path.size()&&(a_path[0]=='/')?true:false);
  }
}

inline void tolowercase(std::string& a_string){
  for(std::string::iterator it=a_string.begin();it!=a_string.end();++it) {
    char c = *it;
    *it = ((c) >= 'A' && (c) <= 'Z' ?  c - 'A' + 'a' : c);
  }
}

inline void touppercase(std::string& a_string){
  for(std::string::iterator it=a_string.begin();it!=a_string.end();++it) {
    char c = *it;
    *it = ((c) >= 'a' && (c) <= 'z' ?  c - 'a' + 'A' : c);
  }
}

enum what { leading, trailing, both };

inline void strip(std::string& a_string,what a_type = both,char a_char = ' '){
  std::string::size_type l = a_string.length();;
  if(l==0) return;

  switch ( a_type ) {
  case leading:{
    std::string::size_type i;
    char* pos = (char*)a_string.c_str();
    for(i=0;i<l;i++,pos++) {
      if(*pos!=a_char) {
        a_string = a_string.substr(i,l-i);
        return;
      }
    }
    }break;
  case trailing:{
    std::string::size_type i;
    char* pos = (char*)a_string.c_str();
    pos += (l-1);
    for(i=l-1;;i--,pos--) {
      if(*pos!=a_char) {
        a_string = a_string.substr(0,i+1);
        return;
      }
    }
    }break;
  case both:
    strip(a_string,leading,a_char);
    strip(a_string,trailing,a_char);
    break;
  //default:break;
  }
}

inline std::string strp(const std::string& a_string,what a_type = both,char a_char = ' '){
  std::string s(a_string);
  strip(s,a_type,a_char);
  return s;
}


inline std::string to_xml(const std::string& a_string){
  // > : &lt;
  // < : &gt;
  // & : &amp;
  // " : &quot;
  // ' : &apos;
  std::string s = a_string;
  replace(s,"<","&lt;");
  replace(s,">","&gt;");
  replace(s,"&","&amp;");
  replace(s,"\"","&quot;");
  replace(s,"'","&apos;");
  return s;
}

inline std::string to_string(const std::vector<std::string>& a_strings,const std::string& a_sep){
  unsigned int number = a_strings.size();
  if(number<=0) return "";
  std::string result;
  number--;
  for(unsigned int index=0;index<number;index++) {
    result += a_strings[index];
    result += a_sep;
  }
  result += a_strings[number];
  return result;
}

inline std::string to_string(unsigned int a_linen,const char* a_lines[]) {
  std::string s;
  for(unsigned int index=0;index<a_linen;index++) {
    s += std::string(a_lines[index]);
    s += "\n";
  }
  return s;
}

inline bool to(const std::string& a_string,bool& a_value){
  if(  (a_string=="1")
     ||(a_string=="true")||(a_string=="TRUE")||(a_string=="True")
     ||(a_string=="yes")||(a_string=="YES")||(a_string=="Yes")
     ||(a_string=="on")||(a_string=="ON")||(a_string=="On")
     ){
    a_value = true;
    return true;
  } else if((a_string=="0")
          ||(a_string=="false")||(a_string=="FALSE")||(a_string=="False")
          ||(a_string=="no")||(a_string=="NO")||(a_string=="No")
          ||(a_string=="off")||(a_string=="OFF")||(a_string=="Off")
          ){
    a_value = false;
    return true;
  } else {
    a_value = false;
    return false;
  }
}

inline std::vector<std::string> to(int a_argc,char** a_argv) {
  std::vector<std::string> v;
  for(int index=0;index<a_argc;index++) v.push_back(a_argv[index]);
  return v;
}

}

#include <sstream>

namespace Slash {

template <class T>
inline bool to(const std::string& a_s,T& a_v) {
  std::istringstream strm(a_s.c_str());
  std::istringstream::pos_type pb = strm.tellg();
  strm >> a_v;
  if(strm.fail()) {a_v = T();return false;}
  if(strm.eof()) return true; //can't check size.
  std::istringstream::pos_type pe = strm.tellg();
  if((pe-pb)!=a_s.size()) {a_v = 0;return false;}
  return true;
}

template <class T>
inline std::string to(const T& a_v) {
  std::ostringstream strm;
  strm << a_v;
  return strm.str();
}

}

#endif
