#ifndef Slash_Tools_system_h
#define Slash_Tools_system_h

#include <string>
#include <cstdlib>
#include "cstring.h"
#include "smanip.h"

namespace Slash {

inline bool isenv(const std::string& a_string){
  const char* env = ::getenv(a_string.c_str());
  return (env?true:false);
}

inline bool getenv(const std::string& a_string,std::string& a_value){
  const char* env = ::getenv(a_string.c_str());
  if(env) {
    a_value = std::string(env?env:"");
    return true;
  } else {
    a_value = "";
    return false;
  }
}

inline bool putenv(const std::string& a_env,const std::string& a_value){
  std::string value = a_env+"="+a_value;
  if(::putenv(Slash::strdup(value.c_str()))) return false;
  //check:
  std::string s;
  if(!getenv(a_env,s)) return false;
  if(s!=a_value) return false;
  return true;
}

inline bool check_getenv(std::ostream& a_out,const std::string& a_new,const std::string& a_old,std::string& a_env){
  if(Slash::getenv(a_new,a_env)) return true;
  if(Slash::getenv(a_old,a_env)) {
    a_out << "Environment variable " << sout(a_old) << " is deprecated."
          << " Use " << sout(a_new) << " instead."
          << std::endl;
    return true;
  }
  return false;
}

inline int execute(const std::string& a_string) {
  return ::system(a_string.c_str());
}

inline std::string file_separator(){
#ifdef WIN32
  return "\\";
#else
  return "/";
#endif
}

inline std::string path_separator(){
#ifdef WIN32
  return ";";
#else
  return ":";
#endif
}

inline bool env_append(const std::string& a_list,const std::string& a_value,const std::string& a_sep){
  std::string old_value;
  if(Slash::isenv(a_list)) {
    if(!Slash::getenv(a_list,old_value)) return false;
  }

  if(old_value.size()) {
    std::vector<std::string> ws;
    Slash::words(old_value,a_sep,false,ws);
    // Remove existing one, so that a_value be put at end.
    Slash::remove(ws,a_value);
    old_value = Slash::to_string(ws,a_sep);
  }

  std::string new_value = old_value;
  if(new_value.size()) new_value += a_sep;
  new_value += a_value;
  if(!putenv(a_list,new_value)) return false;

  return true;
}

inline bool env_path_append(const std::string& a_paths,const std::string& a_path){
  return env_append(a_paths,a_path,path_separator());
}

inline bool expand_vars(std::string& a_string){
  std::string::size_type dollar;
  while((dollar=a_string.find('$'))!=std::string::npos){
    std::string::size_type slash = a_string.find('/',dollar+1);
    std::string::size_type back_slash = a_string.find('\\',dollar+1);
    std::string::size_type pos = std::string::npos; 
    if(slash!=std::string::npos) {
      if(back_slash!=std::string::npos) {
        pos = slash<back_slash?slash:back_slash;
      } else {
        pos = slash;
      }
    } else {
      if(back_slash!=std::string::npos) {
        pos = back_slash;
      } else {
        pos = std::string::npos; 
      }
    }
    std::string env;
    if(pos==std::string::npos) {
      env = a_string.substr(dollar+1,a_string.length()-(dollar+1));
    } else {
      //     abc$xxx/ef
      //     0  3   7 9
      env = a_string.substr(dollar+1,pos-(dollar+1));
    }
    char* val = ::getenv(env.c_str());
    if(!val) return false; // Stop looping if env variable not found.
    std::string value = a_string.substr(0,dollar);
    value += val;
    if(pos!=std::string::npos)
      value += a_string.substr(pos,a_string.length()-pos);
    a_string = value;
  }
  return true;
}

inline std::string file_name(const std::string& a_path) {
  std::string fname = a_path;
  expand_vars(fname);
#ifdef WIN32
  replace(fname,"/","\\");
  replace(fname,"\"","");
#endif
  return fname;
}

inline bool web_get(const std::string& a_path){
  std::string cmd;
  if(Slash::getenv("OSC_WGET",cmd)) {
    cmd += " "+a_path;
  } else {
#ifdef WIN32
    cmd = "C:\\cygwin\\bin\\curl.exe -o";
#else
#ifdef __APPLE__
    cmd = "curl -o";
#else  // Linux
    cmd = "wget -O";
#endif
#endif
    cmd += " "+base_name(a_path)+" "+a_path;
  }
  ::system(cmd.c_str());
  return true;
}

}

#endif
